package com.yeory.gv.util;

import java.lang.ref.WeakReference;

import android.graphics.drawable.BitmapDrawable;
import android.os.AsyncTask;
import android.widget.ImageView;

import com.yeory.gv.util.ImageDownloader.Mode;
/**
 * =============================================================================
/            Ʈ :   GridView_Test
/            ȭ     :   BitmapDrawableDownloaderTask.java
/                   :   
/                   :   http://code.google.com/p/android-imagedownloader/
/            Ư   :
/       The actual AsyncTask that will asynchronously download the image.
/-----------------------------------------------------------------------------
/                                				                     
/-----------------------------------------------------------------------------
/           	(ۼ)                 		                  
/   ----------     	--------------------------       -------------------------
/   2010. 07. 19.      	Gilles Debunne         				 ۼ                      
/   2010. 11. 09.      翭<jyeory@gmail.com>        		κ                       
/==============================================================================
 */
class BitmapDrawableDownloaderTask extends AsyncTask<String, Void, BitmapDrawable> {
    private String url;
    private final WeakReference<ImageView> imageViewReference;
    private ImageDownloader downloader = null;
    private ImageCache iCache;

    public BitmapDrawableDownloaderTask(ImageView imageView, ImageDownloader downloader) {
        imageViewReference = new WeakReference<ImageView>(imageView);
        this.downloader = downloader;
    }
    
    public BitmapDrawableDownloaderTask(ImageView imageView, ImageCache cache) {
        imageViewReference = new WeakReference<ImageView>(imageView);
        this.iCache = cache;
    }

    /**
     * Actual download method.
     */
    @Override
    protected BitmapDrawable doInBackground(String... params) {
        setUrl(params[0]);
        return downloader.getBitmapDrawable(getUrl());
    }

    /**
     * Once the image is downloaded, associates it to the imageView
     */
    @Override
    protected void onPostExecute(BitmapDrawable bitmap) {
        if (isCancelled()) {
            bitmap = null;
        }

        downloader.addBitmapToCache(getUrl(), bitmap);

        if (imageViewReference != null) {
            ImageView imageView = imageViewReference.get();
            BitmapDrawableDownloaderTask bitmapDownloaderTask = ImageDownloader.getBitmapDrawableDownloaderTask(imageView);
            // Change bitmap only if this process is still associated with it
            // Or if we don't use any bitmap to task association (NO_DOWNLOADED_DRAWABLE mode)
            if ((this == bitmapDownloaderTask) || (downloader.mode != Mode.CORRECT)) {
                imageView.setImageBitmap(bitmap.getBitmap());
            }
        }
    }

	public void setUrl(String url) {
		this.url = url;
	}

	public String getUrl() {
		return url;
	}
}