package com.yeory.gv.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.entity.BufferedHttpEntity;

import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.http.AndroidHttpClient;
import android.util.Log;
import android.widget.ImageView;

import com.yeory.gv.common.CommonResources;

public class ImageCache {
	private AndroidHttpClient client = AndroidHttpClient.newInstance("Android");
	private HttpGet httpRequest = null; 
	private int statusCode;
	private HttpResponse response = null; 
	private BufferedHttpEntity bufHttpEntity = null; 
	private BitmapDrawableDownloaderTask task;
    private DownloadedDrawable downloadedDrawable;
    private InputStream is = null;
    private BitmapDrawable bd;
    private WeakReference<BitmapDrawable> ref;
    private List<ImageCallback> callbacks;
    public String url;
    
    private File f;
    private FileOutputStream fo;
    private byte[] buffer = new byte[256];
    private int size;
    

    /**
     * =============================================================================
     *            Ʈ :   GridView_Example
     *            ȭ     :   ImageCache.java
     *                   :   
     *                   :   
     *            Ư   :
     *-----------------------------------------------------------------------------
     *                               				                     
     *-----------------------------------------------------------------------------
     *           	(ۼ)                 		                  
     *   ----------     	--------------------------       -------------------------
     *   2011. 1. 31.      翭<jyeory@gmail.com>         	                         
     *==============================================================================
     * 
     * @author JaeYeol Lee
     *  ̽ ̹ ٿε  ó ۾ ؼ մϴ.
     */
    public static interface ImageCallback {
        /**
         * ̹ ٿε Ϸ  Ʒ ޼带  ۾ ̾ ϴ.
         * @param image
         * @param url
         */
        void onImageLoaded(BitmapDrawable image, String url);
    }
    
    private static ImageCache _instance = null;
    
    /**
     * Gets the singleton instance of the ImageCache.
     * @return The ImageCache.
     */
    public synchronized static ImageCache getInstance() {
        if (_instance == null) {
            _instance = new ImageCache();
        }
        return _instance;
    }

    private static final long CACHE_TIMEOUT = 30000;
    private final Object _lock = new Object();
    private HashMap<String, Object> _cache;
    private HashMap<String, Object> _callbacks;

    private ImageCache() {
        _cache = new HashMap<String, Object>();
        _callbacks = new HashMap<String, Object>();
    }

    private String getHash(String url) {
        try {
            MessageDigest digest = MessageDigest.getInstance("MD5");
            digest.update(url.getBytes());
            return new BigInteger(digest.digest()).toString(16);
        } catch (NoSuchAlgorithmException ex) {
            // this should never happen, but just to make sure return the url
            return url;
        }
    }

    @SuppressWarnings("unchecked")
	private BitmapDrawable bmdFromCache(String url){
    	Log.w("check", "cache size : "+_cache.size());
    	String hash = getHash(url);
    	BitmapDrawable bmd = null;
        synchronized (_lock) {
            if (_cache.containsKey(hash)) {
				ref = (WeakReference<BitmapDrawable>) _cache.get(url);
                if ( ref != null ) {
                    bmd = (BitmapDrawable) ref.get();
                    if (bmd == null)
                        _cache.remove(url);
                }
            }
        }
        return bmd;
    }
    
    private synchronized BitmapDrawable loadSync(String url) {
        try {
        	String hash = getHash(url);
            bd = bmdFromCache(url);
            f = new File(CommonResources.COMMON_CONTEXT.getCacheDir(), hash);
            boolean timeout = f.lastModified() + CACHE_TIMEOUT < new Date().getTime();
            
            if (bd == null || timeout) {
                if (timeout)
                    f.delete();
                
                if (!f.exists()){
                	httpRequest = new HttpGet(url); 
                    response = client.execute(httpRequest);
                    statusCode = response.getStatusLine().getStatusCode();
                    if (statusCode != HttpStatus.SC_OK) { 
                        Log.w("ImageDownloader", "Error " + statusCode + " while retrieving bitmap from " + url); 
                        return null;
                    }
                    
                    bufHttpEntity = new BufferedHttpEntity(response.getEntity()); 
                    is = bufHttpEntity.getContent();
                    
                    if (f.createNewFile()){
                        fo = new FileOutputStream(f);
                        while ((size = is.read(buffer)) > 0) {
                            fo.write(buffer, 0, size);
                        }
                        fo.flush();
                        fo.close();
                    }
                }
                bd = new BitmapDrawable(BitmapFactory.decodeFile(f.getAbsolutePath()));
                synchronized (_lock) {
                    _cache.put(url, new WeakReference<BitmapDrawable>(bd));
                }
            }
        } catch (Exception ex) {
        	httpRequest.abort();
            Log.e(getClass().getName(), ex.getMessage(), ex);
        }finally{
        	if(is!=null){
        		try {
        			is.close();
        		} catch (IOException e) {
        			e.printStackTrace();
        		}
        	}	
        }
        return bd;
    }
    
    /**
     * Adds this bitmap to the cache.
     * @param bd The newly downloaded bitmap.
     */
    public void addBitmapToCache(String url, BitmapDrawable bd) {
        if (bd != null) {
            synchronized (_cache) {
            	_cache.put(url, bd);
            }
        }
    }
    
    public BitmapDrawable getBitmapDdrawableFromCache(String url){
    	Log.w("check", "url : "+url);
    	return (BitmapDrawable) _cache.get(url);
    }

    /**
     * Loads an image from the passed URL and calls the callback method when
     * the image is done loading.
     * @param url The URL of the target image.
     * @param callback A ImageCallback object to pass the loaded image. If null,
     * the image will only be pre-loaded into the cache.
     * @param context The context of the new Drawable image.
     */
    @SuppressWarnings("unchecked")
	public void loadAsync(final String url, final ImageCallback callback, final ImageView imageView) {
        final String hash = getHash(url);
        
        synchronized (_lock) {
            callbacks = (List<ImageCallback>) _callbacks.get(hash);
            if (callbacks != null) {
                if (callback != null)
                    callbacks.add(callback);
                return;
            }
            
            callbacks = new ArrayList<ImageCallback>();
            if (callback != null)
                callbacks.add(callback);
            _callbacks.put(hash, callbacks);
        }
        
        task = new BitmapDrawableDownloaderTask(imageView, ImageCache.this);
        downloadedDrawable = new DownloadedDrawable(task);
        imageView.setImageDrawable(downloadedDrawable);
        imageView.setMinimumHeight(156);
        task.execute(url);
    }
    
    public BitmapDrawable getBitmapDrawable(final String url){
    	return loadSync(url);
    }

    /**
     * @param imageView Any imageView
     * @return Retrieve the currently active download task (if any) associated with this imageView.
     * null if there is no such task.
     */
    public static BitmapDrawableDownloaderTask getBitmapDrawableDownloaderTask(ImageView imageView) {
        if (imageView != null) {
            Drawable drawable = imageView.getDrawable();
            if (drawable instanceof DownloadedDrawable) {
                DownloadedDrawable downloadedDrawable = (DownloadedDrawable)drawable;
                return downloadedDrawable.getBitmapDownloaderTask();
            }
        }
        return null;
    }
    
    /**
     * A fake Drawable that will be attached to the imageView while the download is in progress.
     *
     * <p>Contains a reference to the actual download task, so that a download task can be stopped
     * if a new binding is required, and makes sure that only the last started download process can
     * bind its result, independently of the download finish order.</p>
     */
    static class DownloadedDrawable extends ColorDrawable {
        private final WeakReference<BitmapDrawableDownloaderTask> bitmapDownloaderTaskReference;

        public DownloadedDrawable(BitmapDrawableDownloaderTask bitmapDownloaderTask) {
            super(Color.BLACK);
            bitmapDownloaderTaskReference = new WeakReference<BitmapDrawableDownloaderTask>(bitmapDownloaderTask);
        }

        public BitmapDrawableDownloaderTask getBitmapDownloaderTask() {
            return bitmapDownloaderTaskReference.get();
        }
    }
}