package com.yeory.gv.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.SoftReference;
import java.lang.ref.WeakReference;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.entity.BufferedHttpEntity;
import org.apache.http.impl.client.DefaultHttpClient;

import android.content.Context;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.http.AndroidHttpClient;
import android.os.Handler;
import android.util.Log;
import android.widget.ImageView;

import com.yeory.gv.common.CommonResources;

/**
 * =============================================================================
/            Ʈ :   GridView_Test
/            ȭ     :   ImageDownloader.java
/                   :   
/                   :   http://code.google.com/p/android-imagedownloader/
/            Ư   : 
/-----------------------------------------------------------------------------
/                                				                     
/-----------------------------------------------------------------------------
/           	(ۼ)                 		                  
/   ----------     	--------------------------       -------------------------
/   2010. 11. 9.      翭<jyeory@gmail.com>         		κ ߰                      
/==============================================================================
 */

public class ImageDownloader {
	private static final String LOG_TAG = "ImageDownloader";
	private static final long CACHE_TIMEOUT = 3600000;

	private BitmapDrawable bd;
	private BitmapDrawableDownloaderTask task;
	private DownloadedDrawable downloadedDrawable;
	private BufferedHttpEntity bufHttpEntity = null; 
	private FileOutputStream fo;
	private byte[] buffer = new byte[256];
	private int size;

	private int statusCode;
	private InputStream inputStream;

	public enum Mode { NO_ASYNC_TASK, NO_DOWNLOADED_DRAWABLE, CORRECT }
	public Mode mode = Mode.NO_ASYNC_TASK;

	public ImageDownloader(){}

	private static ImageDownloader instance;

	public static ImageDownloader getInstance(){
		if(instance == null) return instance = new ImageDownloader();
		else return instance;
	}

	private String getHash(String url) {
		try {
			MessageDigest digest = MessageDigest.getInstance("MD5");
			digest.update(url.getBytes());
			return new BigInteger(digest.digest()).toString(16);
		} catch (NoSuchAlgorithmException ex) {
			// this should never happen, but just to make sure return the url
			return url;
		}
	}

	/**
	 * Download the specified image from the Internet and binds it to the provided ImageView. The
	 * binding is immediate if the image is found in the cache and will be done asynchronously
	 * otherwise. A null bitmap will be associated to the ImageView if an error occurs.
	 *
	 * @param url The URL of the image to download.
	 * @param imageView The ImageView to bind the downloaded image to.
	 */
	public void download(String url, ImageView imageView) {
		resetPurgeTimer();
		bd = getBitmapFromCache(url);

		if (bd == null) {
			forceDownload(url, imageView);
		} else {
			cancelPotentialDownload(url, imageView);
			imageView.setImageBitmap(bd.getBitmap());
		}
	}

	/*
	 * Same as download but the image is always downloaded and the cache is not used.
	 * Kept private at the moment as its interest is not clear.
       private void forceDownload(String url, ImageView view) {
          forceDownload(url, view, null);
       }
	 */

	/**
	 * Same as download but the image is always downloaded and the cache is not used.
	 * Kept private at the moment as its interest is not clear.
	 */
	private void forceDownload(String url, ImageView imageView) {
		// State sanity: url is guaranteed to never be null in DownloadedDrawable and cache keys.
		if (url == null) {
			imageView.setImageDrawable(null);
			return;
		}

		if (cancelPotentialDownload(url, imageView)) {
			switch (mode) {
			case NO_ASYNC_TASK:
				bd = getBitmapDrawable(url);
				addBitmapToCache(url, bd);
				imageView.setImageBitmap(bd.getBitmap());
				break;

			case NO_DOWNLOADED_DRAWABLE:
				imageView.setMinimumHeight(156);
				task = new BitmapDrawableDownloaderTask(imageView, this);
				task.execute(url);
				break;

			case CORRECT:
				task = new BitmapDrawableDownloaderTask(imageView, this);
				downloadedDrawable = new DownloadedDrawable(task);
				imageView.setImageDrawable(downloadedDrawable);
				imageView.setMinimumHeight(156);
				task.execute(url);
				break;
			}
		}
	}

	/**
	 * Returns true if the current download has been canceled or if there was no download in
	 * progress on this image view.
	 * Returns false if the download in progress deals with the same url. The download is not
	 * stopped in that case.
	 */
	private static boolean cancelPotentialDownload(String url, ImageView imageView) {
		BitmapDrawableDownloaderTask bitmapDrawableDownloaderTask = getBitmapDrawableDownloaderTask(imageView);

		if (bitmapDrawableDownloaderTask != null) {
			String bitmapUrl = bitmapDrawableDownloaderTask.getUrl();
			if ((bitmapUrl == null) || (!bitmapUrl.equals(url))) {
				bitmapDrawableDownloaderTask.cancel(true);
			} else {
				// The same URL is already being downloaded.
				return false;
			}
		}
		return true;
	}

	/**
	 * @param imageView Any imageView
	 * @return Retrieve the currently active download task (if any) associated with this imageView.
	 * null if there is no such task.
	 */
	public static BitmapDrawableDownloaderTask getBitmapDrawableDownloaderTask(ImageView imageView) {
		if (imageView != null) {
			Drawable drawable = imageView.getDrawable();
			if (drawable instanceof DownloadedDrawable) {
				DownloadedDrawable downloadedDrawable = (DownloadedDrawable)drawable;
				return downloadedDrawable.getBitmapDownloaderTask();
			}
		}
		return null;
	}

	private File downloadImage(String urlAddress){
		// AndroidHttpClient is not allowed to be used from the main thread
		final HttpClient client = (mode == Mode.NO_ASYNC_TASK) ? new DefaultHttpClient() : AndroidHttpClient.newInstance("Android");
		final HttpGet getRequest = new HttpGet(urlAddress);
		File f = null;
		if(sSoftFileCache.containsKey(urlAddress)){
			f = sSoftFileCache.get(urlAddress).get();
			if(f == null) sSoftFileCache.remove(urlAddress);
		}

		if(f == null) f = new File(CommonResources.COMMON_CONTEXT.getCacheDir(), getHash(urlAddress));

		boolean timeout = f.lastModified() + CACHE_TIMEOUT < new Date().getTime();
		try {
			if (timeout) f.delete();
			if (!f.exists()){
				HttpResponse response = client.execute(getRequest);
				statusCode = response.getStatusLine().getStatusCode();
				if (statusCode != HttpStatus.SC_OK)
					return null;
				
				try {
					bufHttpEntity = new BufferedHttpEntity(response.getEntity());
					inputStream = bufHttpEntity.getContent();

					if (f.createNewFile()) {
						fo = new FileOutputStream(f);
						while ((size = inputStream.read(buffer)) > 0) {
							fo.write(buffer, 0, size);
						}
						fo.flush();
						fo.close();
					}
					
				} finally {
					if (inputStream != null) {
						inputStream.close();
					}
					bufHttpEntity.consumeContent();
				}
				
			}else{
				Log.w("test", "Image file already exits , urlAddress : "+urlAddress);
			}
			if(!sSoftFileCache.containsKey(urlAddress)) sSoftFileCache.put(urlAddress, new SoftReference<File>(f));
			return f; 

		} catch (IOException e) {
			getRequest.abort();
			e.printStackTrace();
			Log.w(LOG_TAG, "I/O error while retrieving bitmap from " + urlAddress, e);
		} catch (IllegalStateException e) {
//			request.abort();
			e.printStackTrace();
			Log.w(LOG_TAG, "Incorrect URL: " + urlAddress);
		} catch (Exception e) {
			getRequest.abort();
			e.printStackTrace();
			Log.w(LOG_TAG, "Error while retrieving bitmap from " + urlAddress, e);
		} finally {
			if ((client instanceof AndroidHttpClient)) {
				((AndroidHttpClient) client).close();
			}
			f = null;
		}
		return null;
	}

	public void callDownloadImage(String url){
		getBitmapDrawable(url);
	}

	/**
	 * Image ٿ޾ ĳ .
	 * @param url
	 * @return BitmapDrawable
	 */
	synchronized BitmapDrawable getBitmapDrawable(String url) {
		File f = getImageFileFromCache(url);
		if(f == null){
			Log.w("check", url+" need to make cache file.");
			f = downloadImage(url);
			addImageFileToCache(url, f);
		}else{
			Log.w("check", url+" is cached ? _ "+getImageFileFromCache(url));
		}

		return new BitmapDrawable(BitmapFactory.decodeFile(f.getAbsolutePath()));
	}

	/**
	 * A fake Drawable that will be attached to the imageView while the download is in progress.
	 *
	 * <p>Contains a reference to the actual download task, so that a download task can be stopped
	 * if a new binding is required, and makes sure that only the last started download process can
	 * bind its result, independently of the download finish order.</p>
	 */
	static class DownloadedDrawable extends ColorDrawable {
		private final WeakReference<BitmapDrawableDownloaderTask> bitmapDownloaderTaskReference;

		public DownloadedDrawable(BitmapDrawableDownloaderTask bitmapDownloaderTask) {
			super(Color.BLACK);
			bitmapDownloaderTaskReference = new WeakReference<BitmapDrawableDownloaderTask>(bitmapDownloaderTask);
		}

		public BitmapDrawableDownloaderTask getBitmapDownloaderTask() {
			return bitmapDownloaderTaskReference.get();
		}
	}

	public void setMode(Mode mode) {
		this.mode = mode;
		clearCache();
	}


	/*
	 * Cache-related fields and methods.
	 * 
	 * We use a hard and a soft cache. A soft reference cache is too aggressively cleared by the
	 * Garbage Collector.
	 */

	private static final int HARD_CACHE_CAPACITY = 50;
	private static final int DELAY_BEFORE_PURGE = 10 * 1000; // in milliseconds

	// Hard cache, with a fixed maximum capacity and a life duration
	private final HashMap<String, BitmapDrawable> sHardBitmapDrawableCache =
		new LinkedHashMap<String, BitmapDrawable>(HARD_CACHE_CAPACITY / 2, 0.75f, true) {
		private static final long serialVersionUID = 142495182048991343L;

		@Override
		protected boolean removeEldestEntry(LinkedHashMap.Entry<String, BitmapDrawable> eldest) {
			if (size() > HARD_CACHE_CAPACITY) {
				// Entries push-out of hard reference cache are transferred to soft reference cache
				sSoftBitmapDrawableCache.put(eldest.getKey(), new SoftReference<BitmapDrawable>(eldest.getValue()));
				return true;
			} else
				return false;
		}
	};

	// Soft cache for bitmaps kicked out of hard cache
	private final static ConcurrentHashMap<String, SoftReference<BitmapDrawable>> sSoftBitmapDrawableCache =
		new ConcurrentHashMap<String, SoftReference<BitmapDrawable>>(HARD_CACHE_CAPACITY / 2);

	private final HashMap<String, File> sHardFileCache = new LinkedHashMap<String, File>(HARD_CACHE_CAPACITY / 2, 0.75f, true){
		private static final long serialVersionUID = -1492948367709209794L;

		@Override
		protected boolean removeEldestEntry(LinkedHashMap.Entry<String, File> eldest) {
			if (size() > HARD_CACHE_CAPACITY) {
				// Entries push-out of hard reference cache are transferred to soft reference cache
				sSoftFileCache.put(eldest.getKey(), new SoftReference<File>(eldest.getValue()));
				return true;
			} else
				return false;
		}
	};
	private final static ConcurrentHashMap<String, SoftReference<File>> sSoftFileCache =
		new ConcurrentHashMap<String, SoftReference<File>>(HARD_CACHE_CAPACITY / 2);

	private final static ConcurrentHashMap<String, Context> sContextMap = new ConcurrentHashMap<String, Context>();

	private final Handler purgeHandler = new Handler();

	private final Runnable purger = new Runnable() {
		public void run() {
			clearCache();
		}
	};

	public void addImageFileToCache(String url, File file){
		if(file != null){
			synchronized (sHardFileCache) {
				sHardFileCache.put(url, file);
			}
		}
	}

	private File getImageFileFromCache(String url) {
		// First try the hard reference cache
		synchronized (sHardFileCache) {
			final File f = sHardFileCache.get(url);
			if (f != null) {
				// Bitmap found in hard cache
				// Move element to first position, so that it is removed last
				sHardFileCache.remove(url);
				sHardFileCache.put(url, f);
				return f;
			}
		}

		// Then try the soft reference cache
		SoftReference<File> fileReference = sSoftFileCache.get(url);
		if (fileReference != null) {
			final File f = fileReference.get();
			if (f != null && f.exists()) {
				// Bitmap found in soft cache
				return f;
			} else {
				// Soft reference has been Garbage Collected
				sSoftFileCache.remove(url);
			}
		}

		return null;
	}

	/**
	 * Adds this bitmap to the cache.
	 * @param bd The newly downloaded bitmap.
	 */
	public void addBitmapToCache(String url, BitmapDrawable bd) {
		if (bd != null) {
			synchronized (sHardBitmapDrawableCache) {
				sHardBitmapDrawableCache.put(url, bd);
			}
		}
	}

	/**
	 * @param hash The URL of the image that will be retrieved from the cache.
	 * @return The cached bitmap or null if it was not found.
	 */
	private BitmapDrawable getBitmapFromCache(String hash) {

		// First try the hard reference cache
		synchronized (sHardBitmapDrawableCache) {
			final BitmapDrawable bd = sHardBitmapDrawableCache.get(hash);
			if (bd != null) {
				// Bitmap found in hard cache
				// Move element to first position, so that it is removed last
				sHardBitmapDrawableCache.remove(hash);
				sHardBitmapDrawableCache.put(hash, bd);
				return bd;
			}
		}

		// Then try the soft reference cache
		SoftReference<BitmapDrawable> bitmapReference = sSoftBitmapDrawableCache.get(hash);
		if (bitmapReference != null) {
			final BitmapDrawable bd = bitmapReference.get();
			if (bd != null) {
				// Bitmap found in soft cache
				return bd;
			} else {
				// Soft reference has been Garbage Collected
				sSoftBitmapDrawableCache.remove(hash);
			}
		}

		return null;
	}

	/**
	 * Clears the image cache used internally to improve performance. Note that for memory
	 * efficiency reasons, the cache will automatically be cleared after a certain inactivity delay.
	 */
	public void clearCache() {
		sHardBitmapDrawableCache.clear();
		sSoftBitmapDrawableCache.clear();
	}

	/**
	 * Allow a new delay before the automatic cache clear is done.
	 */
	private void resetPurgeTimer() {
		purgeHandler.removeCallbacks(purger);
		purgeHandler.postDelayed(purger, DELAY_BEFORE_PURGE);
	}
}